# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: rdm.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'rdm.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\trdm.proto\x12\x0corg.d0sl.rdm\x1a\x1cgoogle/protobuf/struct.proto\"\x7f\n\x11\x42\x61seSemanticValue\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x12\x19\n\x0fnumerical_value\x18\x02 \x01(\x01H\x00\x12.\n\rlogical_value\x18\x03 \x01(\x0e\x32\x15.org.d0sl.rdm.LogicalH\x00\x42\x07\n\x05value\"A\n\x14ListOfSemanticValues\x12)\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x1b.org.d0sl.rdm.SemanticValue\"D\n\x0eSemanticObject\x12\x0b\n\x03iri\x18\x01 \x01(\t\x12%\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"\xeb\x01\n\rSemanticValue\x12-\n\x04type\x18\x01 \x01(\x0e\x32\x1f.org.d0sl.rdm.SemanticValueType\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32\x1f.org.d0sl.rdm.BaseSemanticValueH\x00\x12<\n\x0elist_of_values\x18\x03 \x01(\x0b\x32\".org.d0sl.rdm.ListOfSemanticValuesH\x00\x12\x34\n\x0cobject_value\x18\x04 \x01(\x0b\x32\x1c.org.d0sl.rdm.SemanticObjectH\x00\x42\x05\n\x03val\"\xb3\x01\n\x10SymbolDescriptor\x12&\n\x04type\x18\x01 \x01(\x0e\x32\x18.org.d0sl.rdm.SymbolType\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x32\n\targuments\x18\x03 \x03(\x0e\x32\x1f.org.d0sl.rdm.SemanticValueType\x12\x35\n\x0creturn_value\x18\x04 \x01(\x0e\x32\x1f.org.d0sl.rdm.SemanticValueType\"\xb9\x01\n\tSignature\x12\x0b\n\x03\x66qn\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12=\n\x0b\x64\x65scriptors\x18\x03 \x03(\x0b\x32(.org.d0sl.rdm.Signature.DescriptorsEntry\x1aR\n\x10\x44\x65scriptorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.org.d0sl.rdm.SymbolDescriptor:\x02\x38\x01\")\n\x06\x43\x61ller\x12\x11\n\tmodelName\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\":\n\x12\x43\x61lculationContext\x12$\n\x06\x63\x61ller\x18\x01 \x01(\x0b\x32\x14.org.d0sl.rdm.Caller\"\x17\n\x15ListSignaturesRequest\"D\n\x16ListSignaturesResponse\x12*\n\tlibraries\x18\x01 \x03(\x0b\x32\x17.org.d0sl.rdm.Signature\"\xd0\x01\n\x0b\x43\x61llRequest\x12$\n\x03lib\x18\x01 \x01(\x0b\x32\x17.org.d0sl.rdm.Signature\x12&\n\x04type\x18\x02 \x01(\x0e\x32\x18.org.d0sl.rdm.SymbolType\x12\x0c\n\x04name\x18\x03 \x01(\t\x12)\n\x04\x61rgs\x18\x04 \x03(\x0b\x32\x1b.org.d0sl.rdm.SemanticValue\x12\x32\n\x03\x63tx\x18\x05 \x01(\x0b\x32 .org.d0sl.rdm.CalculationContextH\x00\x88\x01\x01\x42\x06\n\x04_ctx\"8\n\nCallResult\x12*\n\x05value\x18\x01 \x01(\x0b\x32\x1b.org.d0sl.rdm.SemanticValue*(\n\x07Logical\x12\t\n\x05\x46\x41LSE\x10\x00\x12\x08\n\x04TRUE\x10\x01\x12\x08\n\x04NONE\x10\x02*Q\n\x11SemanticValueType\x12\n\n\x06STRING\x10\x00\x12\r\n\tNUMERICAL\x10\x01\x12\x0b\n\x07LOGICAL\x10\x02\x12\x08\n\x04LIST\x10\x03\x12\n\n\x06OBJECT\x10\x04*)\n\nSymbolType\x12\r\n\tPREDICATE\x10\x00\x12\x0c\n\x08\x46UNCTION\x10\x01\x32\xae\x01\n\x12RemoteModelService\x12;\n\x04\x43\x61ll\x12\x19.org.d0sl.rdm.CallRequest\x1a\x18.org.d0sl.rdm.CallResult\x12[\n\x0eListSignatures\x12#.org.d0sl.rdm.ListSignaturesRequest\x1a$.org.d0sl.rdm.ListSignaturesResponseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'rdm_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SIGNATURE_DESCRIPTORSENTRY']._loaded_options = None
  _globals['_SIGNATURE_DESCRIPTORSENTRY']._serialized_options = b'8\001'
  _globals['_LOGICAL']._serialized_start=1398
  _globals['_LOGICAL']._serialized_end=1438
  _globals['_SEMANTICVALUETYPE']._serialized_start=1440
  _globals['_SEMANTICVALUETYPE']._serialized_end=1521
  _globals['_SYMBOLTYPE']._serialized_start=1523
  _globals['_SYMBOLTYPE']._serialized_end=1564
  _globals['_BASESEMANTICVALUE']._serialized_start=57
  _globals['_BASESEMANTICVALUE']._serialized_end=184
  _globals['_LISTOFSEMANTICVALUES']._serialized_start=186
  _globals['_LISTOFSEMANTICVALUES']._serialized_end=251
  _globals['_SEMANTICOBJECT']._serialized_start=253
  _globals['_SEMANTICOBJECT']._serialized_end=321
  _globals['_SEMANTICVALUE']._serialized_start=324
  _globals['_SEMANTICVALUE']._serialized_end=559
  _globals['_SYMBOLDESCRIPTOR']._serialized_start=562
  _globals['_SYMBOLDESCRIPTOR']._serialized_end=741
  _globals['_SIGNATURE']._serialized_start=744
  _globals['_SIGNATURE']._serialized_end=929
  _globals['_SIGNATURE_DESCRIPTORSENTRY']._serialized_start=847
  _globals['_SIGNATURE_DESCRIPTORSENTRY']._serialized_end=929
  _globals['_CALLER']._serialized_start=931
  _globals['_CALLER']._serialized_end=972
  _globals['_CALCULATIONCONTEXT']._serialized_start=974
  _globals['_CALCULATIONCONTEXT']._serialized_end=1032
  _globals['_LISTSIGNATURESREQUEST']._serialized_start=1034
  _globals['_LISTSIGNATURESREQUEST']._serialized_end=1057
  _globals['_LISTSIGNATURESRESPONSE']._serialized_start=1059
  _globals['_LISTSIGNATURESRESPONSE']._serialized_end=1127
  _globals['_CALLREQUEST']._serialized_start=1130
  _globals['_CALLREQUEST']._serialized_end=1338
  _globals['_CALLRESULT']._serialized_start=1340
  _globals['_CALLRESULT']._serialized_end=1396
  _globals['_REMOTEMODELSERVICE']._serialized_start=1567
  _globals['_REMOTEMODELSERVICE']._serialized_end=1741
# @@protoc_insertion_point(module_scope)
